/*
 * Twitter Bootstrap 4 plugin for AngularJS.
 * Copyright (c) 2016-2019 Rodziu <mateusz.rohde@gmail.com>
 * License: MIT
 */

!(function() {
    'use strict';

    function bsModalController($scope, $element, $attrs, $timeout, $document, $q, $injector, Modal, ModalBackdrop) {
        let backdrop = Modal.config.backdrop;
        const ctrl = this,
            show = function() {
                if (!$element.hasClass('show')) {
                    $element.css({display: 'block'});
                    $element[0].offsetWidth; // force reflow
                    ModalBackdrop.show(backdrop, $element.hasClass('fade')).then(function() {
                        $element.addClass('show');
                    });
                }
            },
            hide = function() {
                if ($element.hasClass('show')) {
                    $element.removeClass('show');
                    let transitionEnded = false;
                    const callback = function() {
                            ModalBackdrop.hide();
                            $element.css({display: ''});
                        },
                        transition = function() {
                            if (!transitionEnded) {
                                $element[0].removeEventListener('transitionend', transition);
                                callback();
                                transitionEnded = true;
                            }
                        };
                    if ($element.hasClass('fade')) {
                        $element[0].addEventListener('transitionend', transition);
                        $timeout(transition, 300);
                    } else {
                        callback();
                    }
                }
            },
            keydown = function(e) {
                if (ctrl.keyboard && e.key === 'Escape') {
                    ctrl.bsModal = false;
                    $scope.$digest();
                }
            };

        let _isOpen;

        ctrl.$scope = $scope; // used in dismiss directive

        ctrl.$onInit = function() {
            if (angular.isUndefined(ctrl.keyboard)) {
                ctrl.keyboard = Modal.config.keyboard;
            }
        };

        ctrl.$onChanges = function(changes) {
            if ('backdrop' in changes) {
                backdrop = this.backdrop === 'static' ? 'static' : this.backdrop;
            }
        };

        ctrl.$doCheck = function() {
            if (_isOpen !== ctrl.bsModal) {
                let ret = ctrl.onBeforeChange({bsModalController: ctrl});
                if (ret !== false) {
                    ret = Modal.config.onBeforeChange(ctrl, $injector);
                }
                if (ret !== false) {
                    _isOpen = ctrl.bsModal;
                    if (_isOpen) {
                        show();
                    } else {
                        hide();
                    }
                } else {
                    ctrl.bsModal = _isOpen;
                }
            }
        };
        /**
         */
        ctrl.$onDestroy = function() {
            if (ctrl.bsModal) {
                hide();
            }
            $document.off('keydown', keydown);
        };
        // backdrop click
        $element.on('click', function(e) {
            if (window.getSelection().type === 'Range') {
                return;
            }
            if (backdrop === true && e.target === $element[0]) { // .modal covers whole page
                ctrl.bsModal = false;
                $scope.$digest();
            }
        });
        // keyboard esc
        $document.on('keydown', keydown);
    }

    function bsModalDirectiveController() {
        return {
            restrict: 'A',
            bindToController: {
                bsModal: '=',
                keyboard: '<?',
                backdrop: '<?',
                onBeforeChange: '&'
            },
            controller: bsModalController
        };
    }

    /**
     * @ngdoc directive
     * @name bsModal
     *
     * @param {expression} bsModal
     * @param {string|boolean} backdrop
     * @param {boolean} keyboard
     */
    angular.module('angularBS.modal').directive('bsModal', bsModalDirectiveController);
}());
